import pandas as pd
from jinja2 import Template
from collections import Counter
class markdown:
    """
    This mark down class is used for generate .md file for IdeaScale Project.


    """
    def __init__(self,csv_file=None):
        self.data_frame=pd.read_csv(csv_file,encoding="utf8")
        self.entity_list=self.data_frame["entity"].tolist()
        self.frequency=self.data_frame["frequency"].tolist()
        self.total_entity=len(self.entity_list)
        self.template=Template("Tesla Ideas - Community Highlights \n \n" 
                               
                               "## Executive Summary \n"
                            
                               '## The Challenge \n'
                                "You've shown your commitment to innovation by providing IdeaScale to your workforce, your customer community, and your collaborators. This expansive IdeaScale Community has provided you with ideas that allow for continuous innovation, but only if you are able to organize them and take action. \n \n \n"
                                
                                'Fundamental to the process of acting on ideas is being able to discover what your community has demonstrated to be popular, salient, and impactful.   Similarly, innovation is what people do, not software or databases, so understanding the Members of your community is essential for driving innovation by assembling teams of people with demonstrated interest and expertise. \n'
                               
                               '### Proposed Solution \n'
                               'IdeaScale is committed to providing the tools that will nurture the entire innovation life cycle of an idea from capture to project completion. In that spirit, we are offering this data analysis service which characterizes the key assets in your Community - your Ideas and Members. \n \n'
                               
                               "With IdeaScale, you have experienced how a platform where everyone involved in this innovation process can provide their input for future directions. The Tesla Ideas community at IdeaScale has a wide variety of participants who suggest Ideas based on their experience and desire of what future innovations should be like. \n \n"
                               
                               "We have built a Natural Language Processing (NLP) system which can categorize and highlight the best ideas and contriburors through machine learning on the content and user interactions.  \n \n"
                               
                               "Through this data management we will be able to provide you highlights of some of the innovative Ideas, which might add value to the company and the society as a whole. These Ideas are direct from both the minds of your employees and people all over the globe who strive for change.\n \n"
                               
                               "In addition to characterizing the Ideas and Members of your community, we also highlight Terms and Topic that are discussed repeatedly, offering you the ability to discover what future project areas are brewing in the minds of all contributors. \n \n"
                               
                               "### Value \n"
                               "The overall purpose of this activity is to provide executives with information that is important to their decision making. With the best Ideas filtered through our system from the Tesla Ideas community, you will be able to get the thoughts and perspective from both the employee and customer point of view. These Ideas if nurtured along with the company vision will help create more value and prestige to your innovative brand. \n"

                               "### Final thoughts & next steps \n"
                               "Ideas can shape lives, through this activity we are reaching global participants and creating an interactive community for their input to help us innovate and provide your prestigious company with relevant information which can help the company for not only improving the current products/processes but also create opportunities to tap into the markets which have all the potential to embrace this change. Let us help you in exploring the future directions for the company through Innovative Ideas and make the future better. \n"
                                
                               "## Highlights \n"
                               
                               "### Topics and Ideas \n"
                               
                               "{{total_entity}} Potentially important terms are in common usage.  These may be important to executive management for product or concept development and enhancement \n \n"
                               
                               "Some notable terms are: \n \n"
                            
                               
                               '{% for data in datas %}  - {{data}} \n {% endfor %}'
                               
                              
                               "\n"
                               
                               "### Commonly used terms \n"
                               "86 Terms are both commonly used and highly specific.  These terms are specific to existing Tesla products or concepts and are frequently discussed in the community. \n \n"
                               
                               "Some notable terms are : \n\n"
                               "TODO \n\n"
                               
                               "### Roles \n\n"
                               "33 Roles that are potentially important from both; company and customer perspective for improvement of products and experiences. \n \n"
                               
                               "Some notable roles are: \n \n"
                               "TODO \n\n"

                               "### Tesla product features \n\n"
                               "Tesla product features that are both significant and commonly discussed in the community. These may be important to the executive management to analyze user experience alignment. \n \n"
                               
                               "Some notable features are: \n\n"
                               
                               "TODO \n\n"
                               
                               "### Top contributors \n\n"
                               
                               "These are the top contributors; based on their reputation and input in the community. These members create and add value to the overall discussions on different Tesla products and concepts. \n \n"
                               
                               "Some of the top contributors are: \n\n"
                               
                               "TODO \n\n"
                               
                               "###  Ranking of Ideas based on Reputation \n\n"
                               
                               "#### A.  Top ranked ideas and their Selection and Implementation status \n\n"
                               
                                "TODO \n\n"
                               
                               "#### B.  Top ranked ideas that have been marked Selected and completed \n\n"
                               
                               "TODO \n\n"
                               
                               "#### C.  Top ranked ideas that have been marked Selected but not completed \n\n"

                               "TODO \n\n"
                               
                               "### Tags in common usage \n\n"
                               
                               "TODO \n\n"
                               
                               "### The Trending Terms in your community over the past month are (TBD):\n\n"
                               "TODO \n\n"
                               
                               "### Novel Terms \n\n"
                               
                               "Based on novelty, some interesting terms are: \n\n"
                               "TODO \n\n"
                               
                               "### Locations \n\n"

                                "Significant Locations mentioned in the community are: \n\n"
                               "TODO \n\n"

                                "### People / Person \n\n"
                               
                               "Significant people mentioned in community discussion are : \n\n"
                                "TODO \n\n"
                               
                               "### Acronyms \n\n"
                               "Some significant Acronyms are: \n\n"
                               )


        # self.save='# Potentially Important Terms \n {{counter}} Potentially important terms are in common usage. These maybe be important to executive \n '
        #                        'management for product or concept development and enhancement.'
        #                         '\n \n'
        #                        'Some notable terms are:'
        #                        '{% for data in datas %} {% for element in data %} <li> {{element}} ({{data[element]}} ideas) {% endfor %} {% endfor %}'
        #



    # def summary(self,column_list):
    #     array_data=[]
    #
    #     for column in column_list:
    #         json_data={}
    #         data=self.data_frame[column].tolist()
    #         json_data[column]=dict(Counter(data))
    #         array_data.append(json_data[column])
    #
    #     self.array_data=array_data
    #     ## get total Num of key
    #     counter=0
    #     for data in array_data:
    #         for key in data:
    #             counter+=1
    #     self.counter=counter
    #

    def write(self):
        """
        This function will create a readme.md file which is generated by the self.template

        """
        with open("readme.md",'w') as f:
            f.write(self.template.render(total_entity=self.total_entity,datas=self.entity_list))



import pandas
def to_tsv(xlsx_file):
    data_frame=pandas.read_excel(xlsx_file,index_col=None)
    # Replace all columns having spaces with underscores
    data_frame.columns = [c.replace(' ', '_') for c in data_frame.columns]

    # Replace all fields having line breaks with space
    data_frame = data_frame.replace('\n', ' ', regex=True)

    # Write dataframe into csv
    data_frame.to_csv(f'{xlsx_file.replace(".xlsx",".tsv")}', sep='\t', encoding='utf-8', index=False, line_terminator='\r\n')



from setuptools import setup, find_packages
import codecs
import os



VERSION = '0.1.0'
DESCRIPTION = 'IdeaScale Package'

# Setting up
setup(
    name="ideascale",
    version=VERSION,
    author="Tien Nguyen",
    author_email="tiennguyenhotel97@gmail.com",
    description=DESCRIPTION,
    packages=find_packages(),
    install_requires=['pandas','openpyxl','xlsxwriter','jinja2'],
    keywords=['python', 'xlsxformatting', 'Mardown'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)



import xlsxwriter
import pandas as pd
import os

class excel_format:

    """
    Excel Format class help to create customized format excel file.
    """
    def __init__(self, csv_name,path=os.getcwd()):
        self.csv_name = csv_name
        self.path = path
        self.convert_csv_to_xlsx()

    def convert_csv_to_xlsx(self):
        """
        This function will take the data from CSV file in the folder
        Converting CSV file into a data frame
        Return workbook and worksheet for further manipulation

        """
        self.data_frame=pd.read_csv(self.csv_name, encoding='utf-8')
        self.row_length,self.column_length=self.data_frame.shape
        writer=pd.ExcelWriter(os.path.join(self.path,self.csv_name.replace('.csv','.xlsx')),engine="xlsxwriter")
        self.data_frame.to_excel(writer,index=False,sheet_name="Sheet1")
        workbook=writer.book
        worksheet=writer.sheets['Sheet1']
        self.workbook=workbook
        self.worksheet=worksheet

    def row_rotation(self,row,angle):
        """
        This function will rotate the row.
        :param row: Row Index Represent in Int
        :param angle: The degree that you want to rotate. Takes in Int
        :return:
        """
        if row==0:
            row_data=self.data_frame.columns.tolist()
            row_data = [str(data).replace("nan", "") for data in row_data]
        else:
            row_data = self.data_frame.iloc[row].tolist()
            row_data = [str(data).replace("nan", "") for data in row_data]
        format=self.workbook.add_format()
        format.set_rotation(angle)

        tuple_row_data=tuple(row_data)



        self.worksheet.write_row(0,0,tuple_row_data,format)
    def row_format(self,row, height,format=None):

        """
        This function provides custom format for the entire row.

        :param row:  Integer
        :param height: Integer
        :param format: workbook Format
        :return:
        """
        if format!=None:

            format=self.workbook.add_format(format)
        self.worksheet.set_row(row,height,format)
    def freeze_panel(self,column,row):
        """
        This will freeze the pannel
        :param column: Int
        :param row: Int
        :return:
        """
        self.worksheet.freeze_panes(column,row)
    def autofilter_whole_sheet(self):
        """
        Run this function will create a filter for the whole sheet
        :return:
        """

        self.worksheet.autofilter(0,0,self.row_length,self.column_length-1)

    def get_column_index(self,column_name):
        """
        This function return the index given column name
        :param column_name: String
        :return: Integer
        """
        column_names = self.data_frame.columns.tolist()
        index = None

        for inde, name in enumerate(column_names):
            if name == column_name:
                index = inde
        return int(index)
    def condition_format_column(self, column_name,formats=None):
        """
        This provides conditional formatting for column
        :param column_name: String
        :param formats: Workbook Format
        :return:
        """
        index=self.get_column_index(column_name)


        for format in formats:

            self.worksheet.conditional_format(1,index,self.row_length,index,format)
    def num_format(self,column_name,format=None):
        """
        This function provides formatting for number

        :param column_name: String
        :param format: Work book format
        :return:
        """
        json_format={
            'num_format':f"{format}"
        }
        format=self.workbook.add_format(json_format)
        index=self.get_column_index(column_name)
        self.worksheet.set_column(index,index,None, format)
    def set_column_width(self,column_name,column_width):
        """
        This function sets up the width column.
        :param column_name: String

        :param column_width: Integer
        :return:
        """
        index=self.get_column_index(column_name)
        self.worksheet.set_column(index,index,column_width)
    def column_width_auto_size(self,column_name):
        """
        This function will automatically adjust your columns width size
        :param column_name: String
        :return:
        """
        data=self.data_frame[column_name].tolist()
        data.append(column_name)

        data=[str(i).replace("nan",",") for i in data]

        len_list=[len(str(i)) for i in data]

        max_len=max(len_list)

        character_value=data[len_list.index(max_len)]

        lower_case_char=0
        uper_case_char=0
        for char in character_value:
            if char.isupper():
                uper_case_char+=1
            else:
                lower_case_char+=1


        index=self.get_column_index(column_name)
        pixl_value=(90/10)*uper_case_char+(80/10)*lower_case_char
        self.worksheet.set_column_pixels(index,index,pixl_value)
    def column_data_validation(self,column_name,list):
        """
        This functions provides Excel Validation given list and column name
        :param column_name: String
        :param list: List
        :return:
        """
        validation_json={
            'validate':'list',
            'source':list
        }
        index=self.get_column_index(column_name)
        self.worksheet.data_validation(1,index,self.row_length,index,validation_json)

    def no_repeat(self,list):
        """
        This functions remove repeated items in a list
        :param list: list
        :return:
        """
        no_repeated_list=[]
        for item in list:
            if item not in no_repeated_list:
                no_repeated_list.append(item)
        return no_repeated_list

    def get_category(self,column_name):
        """
        This function will return set of category in a column with no repeated value
        :param column_name:String
        :return: List
        """

        category_list=self.data_frame[column_name].tolist()
        return self.no_repeat(category_list)
    def category_color_format_generator(self,item_list,color_list,string=True):
        """
        This functions generating a json data for workbook formatting given Item List and Color List
        :param item_list:List
        :param color_list: List
        :param string: Optional whether data is string or int
        :return: Json Object. Workbook Formatting
        """
        format_array=[]
        zip_list=zip(item_list,color_list)
        for item,color in zip_list:
            color_format=self.workbook.add_format({"bg_color":color})
            if string==True:

                json_format={
                    "type":"cell",
                    "criteria":"equal to",
                    "value":f"\"{item}\"",
                    "format":color_format
                }
            else:
                json_format = {
                    "type": "cell",
                    "criteria": "equal to",
                    "value": f"{item}",
                    "format": color_format
                }
            format_array.append(json_format)
        return format_array







    # def combine(self):
    #     formatD={'type': '3_color_scale',
    #                                  'min_color': "red",
    #                                  'mid_color': "yellow",
    #                                  'max_color': "green"}
    #     formats=[formatD]
    #     grey=self.workbook.add_format({'bg_color':'red'})
    #     formatA={'type': 'cell',
    #                 'criteria':'equal to',
    #                 'value': '"TERM_OF_INTEREST"',
    #                 'format':grey
    #              }
    #     formats1=[formatA]
    #     contitional_format={'type':'3_color_scale','min_color': '#C5D9F1',
    #                                     'max_color': '#538ED5'}
    #
    #     self.condition_format_column("frequency_percentile",formats)
    #     self.condition_format_column("system_asserted_tag", formats1)
    #     self.freeze_panel(1,1)
    #     self.row_format(1,50)
    #     self.row_rotation(0,30)
    #     self.autofilter_whole_sheet()
    #     self.workbook.close()








"""
needed function documentation
set_column()
set_rotation(angle)
worksheet.set_row()
"""



